Package: did
Title: Treatment Effects with Multiple Periods and Groups
Version: 2.5.1
Authors@R: c(person("Brantly", "Callaway", email = "brantly.callaway@uga.edu", role = c("aut", "cre")), person("Pedro H. C.", "Sant'Anna", email="pedro.santanna@emory.edu", role = c("aut")))
URL: https://bcallaway11.github.io/did/,
        https://github.com/bcallaway11/did/
Description: The standard Difference-in-Differences (DID) setup involves two periods and two groups -- a treated group and untreated group.  Many applications of DID methods involve more than two periods and have individuals that are treated at different points in time.  This package contains tools for computing average treatment effect parameters in Difference in Differences setups with more than two periods and with variation in treatment timing using the methods developed in Callaway and Sant'Anna (2021) <doi:10.1016/j.jeconom.2020.12.001>.  The main parameters are group-time average treatment effects which are the average treatment effect for a particular group at a particular time.  These can be aggregated into a fewer number of treatment effect parameters, and the package deals with the cases where there is selective treatment timing, dynamic treatment effects, calendar time effects, or combinations of these.  There are also functions for testing the Difference in Differences assumption, and plotting group-time average treatment effects.
Depends: R (>= 4.1.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: BMisc (>= 1.4.4), Matrix, pbapply, ggplot2, DRDID (>= 1.3.0),
        generics, methods, tidyr, fastglm, data.table (>= 1.15.4),
        dreamerr (>= 1.4.0)
VignetteBuilder: knitr
Suggests: rmarkdown, plm, here, knitr, covr, backports, broom, testthat
        (>= 3.0.0), withr, remotes, callr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-08 15:28:03 UTC; bmc43193
Author: Brantly Callaway [aut, cre],
  Pedro H. C. Sant'Anna [aut]
Maintainer: Brantly Callaway <brantly.callaway@uga.edu>
Repository: CRAN
Date/Publication: 2026-07-08 16:20:02 UTC
