# Generated by roxygen2: do not edit by hand

S3method(ggdid,AGGTEobj)
S3method(ggdid,MP)
S3method(glance,AGGTEobj)
S3method(glance,MP)
S3method(nobs,AGGTEobj)
S3method(nobs,MP)
S3method(print,AGGTEobj)
S3method(print,MP)
S3method(summary,AGGTEobj)
S3method(summary,MP)
S3method(summary,MP.TEST)
S3method(tidy,AGGTEobj)
S3method(tidy,MP)
export(AGGTEobj)
export(DIDparams)
export(MP)
export(MP.TEST)
export(aggte)
export(att_gt)
export(build_sim_dataset)
export(compute.aggte)
export(compute.att_gt)
export(compute.att_gt2)
export(conditional_did_pretest)
export(ggdid)
export(glance)
export(gplot)
export(indicator)
export(mboot)
export(pre_process_did)
export(pre_process_did2)
export(process_attgt)
export(reset.sim)
export(sim)
export(splot)
export(test.mboot)
export(tidy)
export(trimmer)
import(data.table)
import(fastglm)
import(ggplot2)
importFrom(BMisc,TorF)
importFrom(BMisc,get_list_element)
importFrom(BMisc,makeBalancedPanel)
importFrom(BMisc,multiplier_bootstrap)
importFrom(BMisc,rhs_vars)
importFrom(BMisc,toformula)
importFrom(DRDID,drdid_panel)
importFrom(DRDID,drdid_rc)
importFrom(DRDID,reg_did_panel)
importFrom(DRDID,reg_did_rc)
importFrom(DRDID,std_ipw_did_panel)
importFrom(DRDID,std_ipw_did_rc)
importFrom(dreamerr,check_set_arg)
importFrom(generics,glance)
importFrom(generics,tidy)
importFrom(methods,as)
importFrom(methods,is)
importFrom(stats,aggregate)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,cov)
importFrom(stats,ecdf)
importFrom(stats,glm)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.pass)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(tidyr,gather)
importFrom(utils,globalVariables)
