% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AGGTEobj.R
\name{print.AGGTEobj}
\alias{print.AGGTEobj}
\title{print.AGGTEobj}
\usage{
\method{print}{AGGTEobj}(x, ...)
}
\arguments{
\item{x}{a \code{AGGTEobj} object}

\item{...}{extra arguments}
}
\description{
prints value of a \code{AGGTEobj} object
}
