% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{glance.AGGTEobj}
\alias{glance.AGGTEobj}
\title{glance model characteristics from AGGTEobj objects}
\usage{
\method{glance}{AGGTEobj}(x, ...)
}
\arguments{
\item{x}{a model of class AGGTEobj produced by the \code{\link[=aggte]{aggte()}} function}

\item{...}{other arguments passed to methods}
}
\description{
glance model characteristics from AGGTEobj objects
}
