% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate_data.R
\name{build_sim_dataset}
\alias{build_sim_dataset}
\title{build_sim_dataset}
\usage{
build_sim_dataset(sp_list, panel = TRUE)
}
\arguments{
\item{sp_list}{A list of simulation parameters.  See \code{reset.sim} to generate
some default values for parameters}

\item{panel}{whether to construct panel data (the default) or repeated
cross sections data}
}
\value{
a data.frame with the following columns
\itemize{
\item G observations group
\item X value of covariate
\item id observation's id
\item cluster observation's cluster (by construction there is no within-cluster correlation)
\item period time period for current observation
\item Y outcome
\item treat whether or not this unit is ever treated
}
}
\description{
A function for building simulated data
}
