% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AGGTEobj.R
\name{summary.AGGTEobj}
\alias{summary.AGGTEobj}
\title{Summary Aggregate Treatment Effect Parameter Objects}
\usage{
\method{summary}{AGGTEobj}(object, ...)
}
\arguments{
\item{object}{an \code{AGGTEobj} object}

\item{...}{other arguments}
}
\description{
A function to summarize aggregated treatment effect parameters.
}
