% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute.att_gt.R
\name{compute.att_gt}
\alias{compute.att_gt}
\title{Compute Group-Time Average Treatment Effects}
\usage{
compute.att_gt(dp)
}
\arguments{
\item{dp}{A DIDparams object}
}
\value{
a list with length equal to the number of groups times the
number of time periods; each element of the list contains an
object that contains group-time average treatment effect as well
as which group it is for and which time period it is for. It also exports
the influence function which is used externally to compute
standard errors.
}
\description{
\code{compute.att_gt} does the main work for computing
multiperiod group-time average treatment effects
}
\keyword{internal}
