% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MP.R
\name{print.MP}
\alias{print.MP}
\title{print.MP}
\usage{
\method{print}{MP}(x, ...)
}
\arguments{
\item{x}{a \code{MP} object}

\item{...}{extra arguments}
}
\description{
prints value of a \code{MP} object
}
