% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{nobs.MP}
\alias{nobs.MP}
\title{Number of unique cross-sectional units in an MP object}
\usage{
\method{nobs}{MP}(object, ...)
}
\arguments{
\item{object}{a model of class MP produced by the \code{\link[=att_gt]{att_gt()}} function}

\item{...}{additional arguments (ignored)}
}
\value{
Integer. The number of unique cross-sectional units in the data.
}
\description{
Number of unique cross-sectional units in an MP object
}
