% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gplot.R
\name{gplot}
\alias{gplot}
\title{gplot}
\usage{
gplot(
  ssresults,
  ylim = NULL,
  xlab = NULL,
  ylab = NULL,
  title = "Group",
  xgap = 1,
  legend = TRUE,
  ref_line = 0,
  theming = TRUE
)
}
\value{
a \code{ggplot2} object
}
\description{
does the heavy lifting for making a plot of an group-time
average treatment effect
}
\keyword{internal}
