% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gplot.R
\name{splot}
\alias{splot}
\title{splot}
\usage{
splot(
  ssresults,
  ylim = NULL,
  xlab = NULL,
  ylab = NULL,
  title = "Group",
  legend = TRUE,
  ref_line = 0,
  theming = TRUE
)
}
\value{
a \code{ggplot2} object
}
\description{
alternative plotting function for the case with selective treatment
timing
}
\keyword{internal}
