% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/imports.R
\docType{package}
\name{did-package}
\alias{did}
\alias{did-package}
\title{Difference in Differences}
\description{
\code{did} implements Difference in Differences with multiple periods and variation in
treatment timing.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://bcallaway11.github.io/did/}
  \item \url{https://github.com/bcallaway11/did/}
}

}
\author{
\strong{Maintainer}: Brantly Callaway \email{brantly.callaway@uga.edu}

Authors:
\itemize{
  \item Brantly Callaway \email{brantly.callaway@uga.edu}
  \item Pedro H. C. Sant'Anna \email{pedro.santanna@emory.edu}
}

}
\keyword{internal}
