% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdid.R
\name{ggdid}
\alias{ggdid}
\title{Plot \code{did} objects using \code{ggplot2}}
\usage{
ggdid(object, ...)
}
\arguments{
\item{object}{either a \code{MP} object or \code{AGGTEobj} object. See \code{help(ggdid.MP)} and \code{help(ggdid.AGGTEobj)}.}

\item{...}{other arguments}
}
\description{
Function to plot objects from the \code{did} package
}
\examples{
\dontrun{
data(mpdta)
out <- att_gt(yname = "lemp",
              gname = "first.treat",
              idname = "countyreal",
              tname = "year",
              xformla = ~1,
              data = mpdta)

# plot all group-time average treatment effects
ggdid(out)

# plot event study aggregation
es <- aggte(out, type = "dynamic")
ggdid(es)
}

}
