% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggdid.R
\name{ggdid.AGGTEobj}
\alias{ggdid.AGGTEobj}
\title{Plot \code{AGGTEobj} objects}
\usage{
\method{ggdid}{AGGTEobj}(
  object,
  ylim = NULL,
  xlab = NULL,
  ylab = NULL,
  title = "",
  xgap = 1,
  legend = TRUE,
  ref_line = 0,
  theming = TRUE,
  ...
)
}
\arguments{
\item{object}{an \code{AGGTEobj} object, as returned by \code{aggte}.}

\item{ylim}{optional y limits for the plot; setting here makes the y limits
the same across different plots}

\item{xlab}{optional x-axis label}

\item{ylab}{optional y-axis label}

\item{title}{optional plot title}

\item{xgap}{optional gap between the labels on the x-axis.  For example,
\code{xgap=3} indicates that the labels should show up for every third
value on the x-axis.  The default is 1.}

\item{legend}{Whether or not to include a legend (which will indicate color
of pre- and post-treatment estimates).  Default is \code{TRUE}.}

\item{ref_line}{A reference line at this value, usually to compare confidence
intervals to 0. Set to NULL to omit.}

\item{theming}{Set to FALSE to skip all theming so you can do it yourself.}

\item{...}{other arguments}
}
\description{
A function to plot \code{AGGTEobj} objects
}
