% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MP.TEST.R
\name{summary.MP.TEST}
\alias{summary.MP.TEST}
\title{summary.MP.TEST}
\usage{
\method{summary}{MP.TEST}(object, ...)
}
\arguments{
\item{object}{an MP.TEST object}

\item{...}{other variables}
}
\description{
print a summary of test results
}
