% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MP.TEST.R
\name{MP.TEST}
\alias{MP.TEST}
\title{MP.TEST}
\usage{
MP.TEST(
  CvM = NULL,
  CvMb = NULL,
  CvMcval = NULL,
  CvMpval = NULL,
  KS = NULL,
  KSb = NULL,
  KScval = NULL,
  KSpval = NULL,
  clustervars = NULL,
  xformla = NULL
)
}
\arguments{
\item{CvM}{Cramer von Mises test statistic}

\item{CvMb}{a vector of bootstrapped Cramer von Mises test statistics}

\item{CvMcval}{CvM critical value}

\item{CvMpval}{p-value for CvM test}

\item{KS}{Kolmogorov-Smirnov test statistic}

\item{KSb}{a vector of bootstrapped KS test statistics}

\item{KScval}{KS critical value}

\item{KSpval}{p-value for KS test}

\item{clustervars}{vector of which variables were clustered on for the test}

\item{xformla}{formla for the X variables used in the test}
}
\description{
An object that holds results from computing pre-test of the
conditional parallel trends assumption
}
