% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute.att_gt2.R
\name{run_att_gt_estimation}
\alias{run_att_gt_estimation}
\title{Run ATT estimation for a given group-time pair}
\usage{
run_att_gt_estimation(g, t, dp2)
}
\arguments{
\item{g}{group of interest (treated group at time t)}

\item{t}{time period}

\item{dp2}{A DIDparams object v2.0}
}
\value{
a list with the gt cell and the results after performing estimation
}
\description{
\code{run_att_gt_estimation} does the main work for computing
multiperiod group-time average treatment effects
}
\keyword{internal}
