% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy.R
\name{glance.MP}
\alias{glance.MP}
\title{glance model characteristics from MP objects}
\usage{
\method{glance}{MP}(x, ...)
}
\arguments{
\item{x}{a model of class MP produced by the \code{\link[=att_gt]{att_gt()}} function}

\item{...}{other arguments passed to methods}
}
\description{
glance model characteristics from MP objects
}
