% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditional_did_pretest.R
\name{indicator}
\alias{indicator}
\title{indicator}
\usage{
indicator(X, u)
}
\arguments{
\item{X}{matrix of X's from the data}

\item{u}{a particular value to compare X's to}
}
\value{
numeric vector
}
\description{
indicator weighting function
}
\examples{
data(mpdta)
dta <- subset(mpdta, year==2007)
X <- model.matrix(~lpop, data=dta)
X <- indicator(X, X[1,])

}
