% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MP.R
\name{summary.MP}
\alias{summary.MP}
\title{summary.MP}
\usage{
\method{summary}{MP}(object, ...)
}
\arguments{
\item{object}{an \code{MP} object}

\item{...}{extra arguments}
}
\description{
prints a summary of a \code{MP} object
}
