% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute.aggte.R
\name{getSE}
\alias{getSE}
\title{Take influence function and return standard errors}
\usage{
getSE(thisinffunc, DIDparams = NULL)
}
\arguments{
\item{thisinffunc}{An influence function}
}
\value{
scalar standard error
}
\description{
Function to take an nx1 influence function and return
a standard error
}
\keyword{internal}
