% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute.aggte.R
\name{wif}
\alias{wif}
\title{Compute extra term in influence function due to estimating weights}
\usage{
wif(keepers, pg, weights.ind, G, group)
}
\arguments{
\item{keepers}{a vector of indices for which group-time average
treatment effects are used to compute a particular aggregated parameter}

\item{pg}{a vector with same length as total number of group-time average
treatment effects that contains the probability of being in particular group}

\item{weights.ind}{additional sampling weights (nx1)}

\item{G}{vector containing which group a unit belongs to (nx1)}

\item{group}{vector of groups}
}
\value{
nxk influence function matrix
}
\description{
A function to compute the extra term that shows up in the
influence function for aggregated treatment effect parameters
due to estimating the weights
}
\keyword{internal}
