% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{citation}
\alias{citation}
\title{citation}
\usage{
citation()
}
\description{
print the citation for the relevant paper
}
\keyword{internal}
